"""
AIOGitHubAPI: AIOGitHubAPI

Asynchronous Python client for the GitHub API
https://github.com/ludeeus/aiogithubapi
"""
from aiogithubapi.github import AIOGitHubAPI as GitHub

from aiogithubapi.common.exceptions import (
    AIOGitHubAPIAuthenticationException,
    AIOGitHubAPIException,
    AIOGitHubAPIRatelimitException,
)



"""
Class object for AIOGitHubAPIRepositoryPullrequest
Documentation: https://docs.github.com/en/rest/reference/pulls#get-a-pull-request

Generated by generate/generate.py - 2020-08-02 10:03:59.857088
"""
from aiogithubapi.objects.base import AIOGitHubAPIBase


class User(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Labels(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def color(self):
        return self.attributes.get("color", "")

    @property
    def default(self):
        return self.attributes.get("default", True)


class MilestoneCreator(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Milestone(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def labels_url(self):
        return self.attributes.get("labels_url", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def number(self):
        return self.attributes.get("number", None)

    @property
    def state(self):
        return self.attributes.get("state", "")

    @property
    def title(self):
        return self.attributes.get("title", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def creator(self):
        return MilestoneCreator(self.attributes.get("creator", {}))

    @property
    def open_issues(self):
        return self.attributes.get("open_issues", None)

    @property
    def closed_issues(self):
        return self.attributes.get("closed_issues", None)

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def closed_at(self):
        return self.attributes.get("closed_at", "")

    @property
    def due_on(self):
        return self.attributes.get("due_on", "")


class Assignee(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Assignees(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class RequestedReviewers(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class RequestedTeams(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def slug(self):
        return self.attributes.get("slug", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def privacy(self):
        return self.attributes.get("privacy", "")

    @property
    def permission(self):
        return self.attributes.get("permission", "")

    @property
    def members_url(self):
        return self.attributes.get("members_url", "")

    @property
    def repositories_url(self):
        return self.attributes.get("repositories_url", "")

    @property
    def parent(self):
        return self.attributes.get("parent", None)


class HeadUser(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class HeadRepoOwner(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class HeadRepoPermissions(AIOGitHubAPIBase):
    @property
    def admin(self):
        return self.attributes.get("admin", False)

    @property
    def push(self):
        return self.attributes.get("push", False)

    @property
    def pull(self):
        return self.attributes.get("pull", True)


class HeadRepo(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def full_name(self):
        return self.attributes.get("full_name", "")

    @property
    def owner(self):
        return HeadRepoOwner(self.attributes.get("owner", {}))

    @property
    def private(self):
        return self.attributes.get("private", False)

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def fork(self):
        return self.attributes.get("fork", False)

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def archive_url(self):
        return self.attributes.get("archive_url", "")

    @property
    def assignees_url(self):
        return self.attributes.get("assignees_url", "")

    @property
    def blobs_url(self):
        return self.attributes.get("blobs_url", "")

    @property
    def branches_url(self):
        return self.attributes.get("branches_url", "")

    @property
    def collaborators_url(self):
        return self.attributes.get("collaborators_url", "")

    @property
    def comments_url(self):
        return self.attributes.get("comments_url", "")

    @property
    def commits_url(self):
        return self.attributes.get("commits_url", "")

    @property
    def compare_url(self):
        return self.attributes.get("compare_url", "")

    @property
    def contents_url(self):
        return self.attributes.get("contents_url", "")

    @property
    def contributors_url(self):
        return self.attributes.get("contributors_url", "")

    @property
    def deployments_url(self):
        return self.attributes.get("deployments_url", "")

    @property
    def downloads_url(self):
        return self.attributes.get("downloads_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def forks_url(self):
        return self.attributes.get("forks_url", "")

    @property
    def git_commits_url(self):
        return self.attributes.get("git_commits_url", "")

    @property
    def git_refs_url(self):
        return self.attributes.get("git_refs_url", "")

    @property
    def git_tags_url(self):
        return self.attributes.get("git_tags_url", "")

    @property
    def git_url(self):
        return self.attributes.get("git_url", "")

    @property
    def issue_comment_url(self):
        return self.attributes.get("issue_comment_url", "")

    @property
    def issue_events_url(self):
        return self.attributes.get("issue_events_url", "")

    @property
    def issues_url(self):
        return self.attributes.get("issues_url", "")

    @property
    def keys_url(self):
        return self.attributes.get("keys_url", "")

    @property
    def labels_url(self):
        return self.attributes.get("labels_url", "")

    @property
    def languages_url(self):
        return self.attributes.get("languages_url", "")

    @property
    def merges_url(self):
        return self.attributes.get("merges_url", "")

    @property
    def milestones_url(self):
        return self.attributes.get("milestones_url", "")

    @property
    def notifications_url(self):
        return self.attributes.get("notifications_url", "")

    @property
    def pulls_url(self):
        return self.attributes.get("pulls_url", "")

    @property
    def releases_url(self):
        return self.attributes.get("releases_url", "")

    @property
    def ssh_url(self):
        return self.attributes.get("ssh_url", "")

    @property
    def stargazers_url(self):
        return self.attributes.get("stargazers_url", "")

    @property
    def statuses_url(self):
        return self.attributes.get("statuses_url", "")

    @property
    def subscribers_url(self):
        return self.attributes.get("subscribers_url", "")

    @property
    def subscription_url(self):
        return self.attributes.get("subscription_url", "")

    @property
    def tags_url(self):
        return self.attributes.get("tags_url", "")

    @property
    def teams_url(self):
        return self.attributes.get("teams_url", "")

    @property
    def trees_url(self):
        return self.attributes.get("trees_url", "")

    @property
    def clone_url(self):
        return self.attributes.get("clone_url", "")

    @property
    def mirror_url(self):
        return self.attributes.get("mirror_url", "")

    @property
    def hooks_url(self):
        return self.attributes.get("hooks_url", "")

    @property
    def svn_url(self):
        return self.attributes.get("svn_url", "")

    @property
    def homepage(self):
        return self.attributes.get("homepage", "")

    @property
    def language(self):
        return self.attributes.get("language", None)

    @property
    def forks_count(self):
        return self.attributes.get("forks_count", None)

    @property
    def stargazers_count(self):
        return self.attributes.get("stargazers_count", None)

    @property
    def watchers_count(self):
        return self.attributes.get("watchers_count", None)

    @property
    def size(self):
        return self.attributes.get("size", None)

    @property
    def default_branch(self):
        return self.attributes.get("default_branch", "")

    @property
    def open_issues_count(self):
        return self.attributes.get("open_issues_count", None)

    @property
    def is_template(self):
        return self.attributes.get("is_template", True)

    @property
    def topics(self):
        return self.attributes.get("topics", [])

    @property
    def has_issues(self):
        return self.attributes.get("has_issues", True)

    @property
    def has_projects(self):
        return self.attributes.get("has_projects", True)

    @property
    def has_wiki(self):
        return self.attributes.get("has_wiki", True)

    @property
    def has_pages(self):
        return self.attributes.get("has_pages", False)

    @property
    def has_downloads(self):
        return self.attributes.get("has_downloads", True)

    @property
    def archived(self):
        return self.attributes.get("archived", False)

    @property
    def disabled(self):
        return self.attributes.get("disabled", False)

    @property
    def visibility(self):
        return self.attributes.get("visibility", "")

    @property
    def pushed_at(self):
        return self.attributes.get("pushed_at", "")

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def permissions(self):
        return HeadRepoPermissions(self.attributes.get("permissions", {}))

    @property
    def allow_rebase_merge(self):
        return self.attributes.get("allow_rebase_merge", True)

    @property
    def template_repository(self):
        return self.attributes.get("template_repository", None)

    @property
    def temp_clone_token(self):
        return self.attributes.get("temp_clone_token", "")

    @property
    def allow_squash_merge(self):
        return self.attributes.get("allow_squash_merge", True)

    @property
    def delete_branch_on_merge(self):
        return self.attributes.get("delete_branch_on_merge", True)

    @property
    def allow_merge_commit(self):
        return self.attributes.get("allow_merge_commit", True)

    @property
    def subscribers_count(self):
        return self.attributes.get("subscribers_count", None)

    @property
    def network_count(self):
        return self.attributes.get("network_count", None)


class Head(AIOGitHubAPIBase):
    @property
    def label(self):
        return self.attributes.get("label", "")

    @property
    def ref(self):
        return self.attributes.get("ref", "")

    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def user(self):
        return HeadUser(self.attributes.get("user", {}))

    @property
    def repo(self):
        return HeadRepo(self.attributes.get("repo", {}))


class BaseUser(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class BaseRepoOwner(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class BaseRepoPermissions(AIOGitHubAPIBase):
    @property
    def admin(self):
        return self.attributes.get("admin", False)

    @property
    def push(self):
        return self.attributes.get("push", False)

    @property
    def pull(self):
        return self.attributes.get("pull", True)


class BaseRepo(AIOGitHubAPIBase):
    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def full_name(self):
        return self.attributes.get("full_name", "")

    @property
    def owner(self):
        return BaseRepoOwner(self.attributes.get("owner", {}))

    @property
    def private(self):
        return self.attributes.get("private", False)

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def description(self):
        return self.attributes.get("description", "")

    @property
    def fork(self):
        return self.attributes.get("fork", False)

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def archive_url(self):
        return self.attributes.get("archive_url", "")

    @property
    def assignees_url(self):
        return self.attributes.get("assignees_url", "")

    @property
    def blobs_url(self):
        return self.attributes.get("blobs_url", "")

    @property
    def branches_url(self):
        return self.attributes.get("branches_url", "")

    @property
    def collaborators_url(self):
        return self.attributes.get("collaborators_url", "")

    @property
    def comments_url(self):
        return self.attributes.get("comments_url", "")

    @property
    def commits_url(self):
        return self.attributes.get("commits_url", "")

    @property
    def compare_url(self):
        return self.attributes.get("compare_url", "")

    @property
    def contents_url(self):
        return self.attributes.get("contents_url", "")

    @property
    def contributors_url(self):
        return self.attributes.get("contributors_url", "")

    @property
    def deployments_url(self):
        return self.attributes.get("deployments_url", "")

    @property
    def downloads_url(self):
        return self.attributes.get("downloads_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def forks_url(self):
        return self.attributes.get("forks_url", "")

    @property
    def git_commits_url(self):
        return self.attributes.get("git_commits_url", "")

    @property
    def git_refs_url(self):
        return self.attributes.get("git_refs_url", "")

    @property
    def git_tags_url(self):
        return self.attributes.get("git_tags_url", "")

    @property
    def git_url(self):
        return self.attributes.get("git_url", "")

    @property
    def issue_comment_url(self):
        return self.attributes.get("issue_comment_url", "")

    @property
    def issue_events_url(self):
        return self.attributes.get("issue_events_url", "")

    @property
    def issues_url(self):
        return self.attributes.get("issues_url", "")

    @property
    def keys_url(self):
        return self.attributes.get("keys_url", "")

    @property
    def labels_url(self):
        return self.attributes.get("labels_url", "")

    @property
    def languages_url(self):
        return self.attributes.get("languages_url", "")

    @property
    def merges_url(self):
        return self.attributes.get("merges_url", "")

    @property
    def milestones_url(self):
        return self.attributes.get("milestones_url", "")

    @property
    def notifications_url(self):
        return self.attributes.get("notifications_url", "")

    @property
    def pulls_url(self):
        return self.attributes.get("pulls_url", "")

    @property
    def releases_url(self):
        return self.attributes.get("releases_url", "")

    @property
    def ssh_url(self):
        return self.attributes.get("ssh_url", "")

    @property
    def stargazers_url(self):
        return self.attributes.get("stargazers_url", "")

    @property
    def statuses_url(self):
        return self.attributes.get("statuses_url", "")

    @property
    def subscribers_url(self):
        return self.attributes.get("subscribers_url", "")

    @property
    def subscription_url(self):
        return self.attributes.get("subscription_url", "")

    @property
    def tags_url(self):
        return self.attributes.get("tags_url", "")

    @property
    def teams_url(self):
        return self.attributes.get("teams_url", "")

    @property
    def trees_url(self):
        return self.attributes.get("trees_url", "")

    @property
    def clone_url(self):
        return self.attributes.get("clone_url", "")

    @property
    def mirror_url(self):
        return self.attributes.get("mirror_url", "")

    @property
    def hooks_url(self):
        return self.attributes.get("hooks_url", "")

    @property
    def svn_url(self):
        return self.attributes.get("svn_url", "")

    @property
    def homepage(self):
        return self.attributes.get("homepage", "")

    @property
    def language(self):
        return self.attributes.get("language", None)

    @property
    def forks_count(self):
        return self.attributes.get("forks_count", None)

    @property
    def stargazers_count(self):
        return self.attributes.get("stargazers_count", None)

    @property
    def watchers_count(self):
        return self.attributes.get("watchers_count", None)

    @property
    def size(self):
        return self.attributes.get("size", None)

    @property
    def default_branch(self):
        return self.attributes.get("default_branch", "")

    @property
    def open_issues_count(self):
        return self.attributes.get("open_issues_count", None)

    @property
    def is_template(self):
        return self.attributes.get("is_template", True)

    @property
    def topics(self):
        return self.attributes.get("topics", [])

    @property
    def has_issues(self):
        return self.attributes.get("has_issues", True)

    @property
    def has_projects(self):
        return self.attributes.get("has_projects", True)

    @property
    def has_wiki(self):
        return self.attributes.get("has_wiki", True)

    @property
    def has_pages(self):
        return self.attributes.get("has_pages", False)

    @property
    def has_downloads(self):
        return self.attributes.get("has_downloads", True)

    @property
    def archived(self):
        return self.attributes.get("archived", False)

    @property
    def disabled(self):
        return self.attributes.get("disabled", False)

    @property
    def visibility(self):
        return self.attributes.get("visibility", "")

    @property
    def pushed_at(self):
        return self.attributes.get("pushed_at", "")

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def permissions(self):
        return BaseRepoPermissions(self.attributes.get("permissions", {}))

    @property
    def allow_rebase_merge(self):
        return self.attributes.get("allow_rebase_merge", True)

    @property
    def template_repository(self):
        return self.attributes.get("template_repository", None)

    @property
    def temp_clone_token(self):
        return self.attributes.get("temp_clone_token", "")

    @property
    def allow_squash_merge(self):
        return self.attributes.get("allow_squash_merge", True)

    @property
    def delete_branch_on_merge(self):
        return self.attributes.get("delete_branch_on_merge", True)

    @property
    def allow_merge_commit(self):
        return self.attributes.get("allow_merge_commit", True)

    @property
    def subscribers_count(self):
        return self.attributes.get("subscribers_count", None)

    @property
    def network_count(self):
        return self.attributes.get("network_count", None)


class Base(AIOGitHubAPIBase):
    @property
    def label(self):
        return self.attributes.get("label", "")

    @property
    def ref(self):
        return self.attributes.get("ref", "")

    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def user(self):
        return BaseUser(self.attributes.get("user", {}))

    @property
    def repo(self):
        return BaseRepo(self.attributes.get("repo", {}))


class MergedBy(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class AIOGitHubAPIRepositoryPullrequest(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def diff_url(self):
        return self.attributes.get("diff_url", "")

    @property
    def patch_url(self):
        return self.attributes.get("patch_url", "")

    @property
    def issue_url(self):
        return self.attributes.get("issue_url", "")

    @property
    def commits_url(self):
        return self.attributes.get("commits_url", "")

    @property
    def review_comments_url(self):
        return self.attributes.get("review_comments_url", "")

    @property
    def review_comment_url(self):
        return self.attributes.get("review_comment_url", "")

    @property
    def comments_url(self):
        return self.attributes.get("comments_url", "")

    @property
    def statuses_url(self):
        return self.attributes.get("statuses_url", "")

    @property
    def number(self):
        return self.attributes.get("number", None)

    @property
    def state(self):
        return self.attributes.get("state", "")

    @property
    def locked(self):
        return self.attributes.get("locked", True)

    @property
    def title(self):
        return self.attributes.get("title", "")

    @property
    def user(self):
        return User(self.attributes.get("user", {}))

    @property
    def body(self):
        return self.attributes.get("body", "")

    @property
    def labels(self):
        return [Labels(x) for x in self.attributes.get("labels", [])]

    @property
    def milestone(self):
        return Milestone(self.attributes.get("milestone", {}))

    @property
    def active_lock_reason(self):
        return self.attributes.get("active_lock_reason", "")

    @property
    def created_at(self):
        return self.attributes.get("created_at", "")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at", "")

    @property
    def closed_at(self):
        return self.attributes.get("closed_at", "")

    @property
    def merged_at(self):
        return self.attributes.get("merged_at", "")

    @property
    def merge_commit_sha(self):
        return self.attributes.get("merge_commit_sha", "")

    @property
    def assignee(self):
        return Assignee(self.attributes.get("assignee", {}))

    @property
    def assignees(self):
        return [Assignees(x) for x in self.attributes.get("assignees", [])]

    @property
    def requested_reviewers(self):
        return [
            RequestedReviewers(x)
            for x in self.attributes.get("requested_reviewers", [])
        ]

    @property
    def requested_teams(self):
        return [RequestedTeams(x) for x in self.attributes.get("requested_teams", [])]

    @property
    def head(self):
        return Head(self.attributes.get("head", {}))

    @property
    def base(self):
        return Base(self.attributes.get("base", {}))

    @property
    def author_association(self):
        return self.attributes.get("author_association", "")

    @property
    def draft(self):
        return self.attributes.get("draft", False)

    @property
    def merged(self):
        return self.attributes.get("merged", False)

    @property
    def mergeable(self):
        return self.attributes.get("mergeable", True)

    @property
    def rebaseable(self):
        return self.attributes.get("rebaseable", True)

    @property
    def mergeable_state(self):
        return self.attributes.get("mergeable_state", "")

    @property
    def merged_by(self):
        return MergedBy(self.attributes.get("merged_by", {}))

    @property
    def comments(self):
        return self.attributes.get("comments", None)

    @property
    def review_comments(self):
        return self.attributes.get("review_comments", None)

    @property
    def maintainer_can_modify(self):
        return self.attributes.get("maintainer_can_modify", True)

    @property
    def commits(self):
        return self.attributes.get("commits", None)

    @property
    def additions(self):
        return self.attributes.get("additions", None)

    @property
    def deletions(self):
        return self.attributes.get("deletions", None)

    @property
    def changed_files(self):
        return self.attributes.get("changed_files", None)



"""AIOGitHubAPI: Constants"""
ACCEPT_HEADERS = {
    "base": "application/vnd.github.v3.raw+json",
    "preview": "application/vnd.github.mercy-preview+json",
}

BASE_API_HEADERS = {
    "Accept": ACCEPT_HEADERS["base"],
    "User-Agent": "python/AIOGitHubAPI",
}

BASE_API_URL = "https://api.github.com"


# Status codes
HTTP_STATUS_CODE_OK = 200
HTTP_STATUS_CODE_CREATED = 201
HTTP_STATUS_CODE_ACCEPTED = 202
HTTP_STATUS_CODE_NON_AUTHORITATIVE = 203
HTTP_STATUS_CODE_RATELIMIT = 403
HTTP_STATUS_CODE_TEAPOT = 418

HTTP_STATUS_CODE_GOOD_LIST = [
    HTTP_STATUS_CODE_OK,
    HTTP_STATUS_CODE_CREATED,
    HTTP_STATUS_CODE_ACCEPTED,
    HTTP_STATUS_CODE_NON_AUTHORITATIVE,
]



"""
AIOGitHubAPI: objects.ratelimit

https://developer.github.com/v3/rate_limit/
"""
# pylint: disable=missing-docstring
from datetime import datetime
from aiogithubapi.objects.base import AIOGitHubAPIBase


class AIOGitHubAPIRateLimit(AIOGitHubAPIBase):
    """
    AIOGitHubAPIRateLimit

    Holds information about the current reatelimit status.
    """

    def __init__(self) -> None:
        """Initialize."""
        self.limit = None
        self.remaining = None
        self.reset = None

    @property
    def reset_utc(self) -> None:
        """Return date + time in UTC for next reset."""
        if self.reset is None:
            return None
        return datetime.utcfromtimestamp(int(self.reset))

    def load_from_response_headers(self, headers: dict) -> None:
        """
        Load from response headers.

        :param headers:     A dctionary with the returned headers
        """
        self.limit = headers.get("X-RateLimit-Limit", "0")
        self.remaining = headers.get("X-RateLimit-Remaining", "0")
        self.reset = headers.get("X-RateLimit-Reset", "0")



"""AIOGitHubAPI: AIOGitHubAPI"""
import os
import aiohttp
from aiogithubapi.client import AIOGitHubAPIClient
from aiogithubapi.common.const import ACCEPT_HEADERS
from aiogithubapi.objects.base import AIOGitHubAPIBase
from aiogithubapi.objects.repository import AIOGitHubAPIRepository


class AIOGitHubAPI(AIOGitHubAPIBase):
    """
    AIOGitHubAPI

    This is the main class this is where it all starts.
    """

    _close_session = False

    def __init__(
        self, token: str = None, session: "aiohttp.ClientSession" = None
    ) -> None:
        """
        Initialises a GitHub API client.

        :param session:     aiohttp.ClientSession to be used by this package.
        :param token:       Your GitHub Personal Access Token
                            https://github.com/settings/tokens
        """
        if session is None:
            session = aiohttp.ClientSession()
            self._close_session = True

        if token is None:
            token = os.getenv("GITHUB_TOKEN")

        self.client = AIOGitHubAPIClient(session, token)

    async def __aenter__(self) -> "AIOGitHubAPI":
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close()

    async def get_repo(self, repo: str) -> "AIOGitHubAPIRepository":
        """Retrun AIOGitHubAPIRepository object."""
        _endpoint = f"/repos/{repo}"
        _headers = {"Accept": ACCEPT_HEADERS["preview"]}

        response = await self.client.get(endpoint=_endpoint, headers=_headers)

        return AIOGitHubAPIRepository(self.client, response)

    async def get_org_repos(
        self, org: str, page: int = 1
    ) -> ["AIOGitHubAPIRepository"]:
        """
        Retrun a list of AIOGitHubAPIRepository objects.

        :param org:         The name of the organization
                            Example: "octocat"
        :param page:        The page number you want to fetch
                            Default: 1
        """
        _enpoint = f"/orgs/{org}/repos?page={str(page)}"
        _params = {"per_page": 100}
        _headers = {"Accept": ACCEPT_HEADERS["preview"]}

        response = await self.client.get(
            endpoint=_enpoint, params=_params, headers=_headers
        )

        return [AIOGitHubAPIRepository(self.client, x) for x in response or []]

    async def get_rate_limit(self) -> dict:
        """Retrun current rate limits."""
        _endpoint = f"/rate_limit"

        await self.client.get(endpoint=_endpoint)
        return self.client.ratelimits.__dict__

    async def render_markdown(self, content: str) -> str:
        """
        Retrun AIOGitHubAPIRepository object.

        :param content:     The content (as markdown) you want to render
        """
        _endpoint = "/markdown/raw"
        _headers = {"Content-Type": "text/plain"}

        return await self.client.post(
            endpoint=_endpoint, headers=_headers, data=content
        )

    async def close(self) -> None:
        """Close open client session."""
        if self.client.session and self._close_session:
            await self.client.session.close()



"""
AIOGitHubAPI: Issue Comment

https://developer.github.com/v3/issues/comments/
"""
# pylint: disable=missing-docstring
from aiogithubapi.objects.base import AIOGitHubAPIBase, AIOGitHubAPIBaseClient
from aiogithubapi.objects.users.user import AIOGitHubAPIUsersUser


class AIOGitHubAPIRepositoryIssueCommentUser(AIOGitHubAPIUsersUser):
    """Issue commment user GitHub API implementation."""

    @property
    def login(self):
        return self.attributes.get("login")

    @property
    def id(self):
        return self.attributes.get("id")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url")

    @property
    def html_url(self):
        return self.attributes.get("html_url")

    @property
    def type(self):
        return self.attributes.get("type")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin")


class AIOGitHubAPIRepositoryIssueComment(AIOGitHubAPIBaseClient):
    """Issue comment GitHub API implementation."""

    def __init__(self, client: "AIOGitHubAPIClient", attributes: dict, repository: str):
        """Initialize."""
        super().__init__(client, attributes)
        self.repository = repository

    @property
    def html_url(self):
        return self.attributes.get("html_url")

    @property
    def id(self):
        return self.attributes.get("id")

    @property
    def created_at(self):
        return self.attributes.get("created_at")

    @property
    def updated_at(self):
        return self.attributes.get("updated_at")

    @property
    def body(self):
        return self.attributes.get("body")

    @property
    def user(self):
        return AIOGitHubAPIRepositoryIssueCommentUser(self.attributes.get("user", {}))

    async def update(self, body: str) -> None:
        """Updates an issue comment."""
        _endpoint = f"/repos/{self.repository}/issues/comments/{self.id}"

        await self.client.post(endpoint=_endpoint, data={"body": body}, jsondata=True)



"""
AIOGitHubAPI: Repository

https://developer.github.com/v3/repos/#get
"""
# pylint: disable=redefined-builtin, missing-docstring, invalid-name, unused-import
from datetime import datetime

from aiogithubapi.common.exceptions import AIOGitHubAPIException
from aiogithubapi.objects.base import AIOGitHubAPIBaseClient
from aiogithubapi.objects.repository.traffic import AIOGitHubAPIRepositoryTraffic
from aiogithubapi.objects.repos.commit import AIOGitHubAPIReposCommit

from aiogithubapi.objects.repository.content import (
    AIOGitHubAPIRepositoryContent,
    AIOGitHubAPIRepositoryTreeContent,
)

from aiogithubapi.objects.repository.issue import (
    AIOGitHubAPIRepositoryIssue,
    AIOGitHubAPIRepositoryIssueComment,
    AIOGitHubAPIRepositoryIssueCommentUser,
)

from aiogithubapi.objects.repository.release import AIOGitHubAPIRepositoryRelease

from aiogithubapi.objects.users.user import AIOGitHubAPIUsersUser


class AIOGitHubAPIRepository(AIOGitHubAPIBaseClient):
    """Repository GitHub API implementation."""

    def __init__(self, client: "AIOGitHubAPIClient", attributes: dict) -> None:
        """Initialise."""
        super().__init__(client, attributes)
        self._last_commit = None
        self.traffic = AIOGitHubAPIRepositoryTraffic(client, attributes)

    @property
    def id(self) -> None:
        return self.attributes.get("id")

    @property
    def name(self) -> None:
        return self.attributes.get("name")

    @property
    def full_name(self) -> None:
        return self.attributes.get("full_name")

    @property
    def pushed_at(self) -> None:
        return datetime.strptime(self.attributes.get("pushed_at"), "%Y-%m-%dT%H:%M:%SZ")

    @property
    def archived(self) -> None:
        return self.attributes.get("archived")

    @property
    def description(self) -> None:
        return self.attributes.get("description")

    @property
    def topics(self) -> None:
        return self.attributes.get("topics")

    @property
    def fork(self) -> None:
        return self.attributes.get("fork")

    @property
    def forks_count(self) -> int:
        return self.attributes.get("forks_count")

    @property
    def default_branch(self) -> None:
        return self.attributes.get("default_branch")

    @property
    def homepage(self) -> str:
        return self.attributes.get("homepage")

    @property
    def stargazers_count(self) -> str:
        return self.attributes.get("stargazers_count")

    @property
    def watchers_count(self) -> str:
        return self.attributes.get("watchers_count")

    @property
    def last_commit(self) -> None:
        if self._last_commit is None:
            self.logger.warning(
                "You need to call .set_last_commit to set this property"
            )
        return self._last_commit

    @property
    def owner(self) -> AIOGitHubAPIUsersUser:
        return AIOGitHubAPIUsersUser(self.attributes.get("owner"))

    async def get_contents(
        self, path: str, ref: str or None = None
    ) -> ["AIOGitHubAPIRepositoryContent"] or "AIOGitHubAPIRepositoryContent":
        """Retrun a list of repository content objects."""
        _endpoint = f"/repos/{self.full_name}/contents/{path}"
        _params = {"path": path}

        if ref is not None:
            _params["ref"] = ref.replace("tags/", "")

        response = await self.client.get(endpoint=_endpoint, params=_params)
        if isinstance(response, list):
            return [AIOGitHubAPIRepositoryContent(x) for x in response]
        return AIOGitHubAPIRepositoryContent(response)

    async def get_tree(
        self, ref: str or None = None
    ) -> ["AIOGitHubAPIRepositoryTreeContent"] or list:
        """Retrun a list of repository tree objects."""
        if ref is None:
            raise AIOGitHubAPIException("Missing ref")
        _endpoint = f"/repos/{self.full_name}/git/trees/{ref}"
        _params = {"recursive": "1"}

        response = await self.client.get(endpoint=_endpoint, params=_params)

        return [
            AIOGitHubAPIRepositoryTreeContent(x, self.full_name, ref)
            for x in response.get("tree", [])
        ]

    async def get_rendered_contents(self, path: str, ref: str or None = None) -> str:
        """Retrun a redered representation of a file."""
        _endpoint = f"/repos/{self.full_name}/contents/{path}"
        _headers = {"Accept": "application/vnd.github.v3.html"}
        _params = {"path": path}

        if ref is not None:
            _params["ref"] = ref.replace("tags/", "")

        return await self.client.get(
            endpoint=_endpoint, params=_params, headers=_headers, returnjson=False
        )

    async def get_releases(
        self, prerelease: bool = False, returnlimit: int = 5
    ) -> ["AIOGitHubAPIRepositoryRelease"] or list:
        """Retrun a list of repository release objects."""
        _endpoint = f"/repos/{self.full_name}/releases"

        response = await self.client.get(endpoint=_endpoint)
        contents = []

        for content in response or []:
            if len(contents) == returnlimit:
                break
            if not prerelease:
                if content.get("prerelease", False):
                    continue
            contents.append(AIOGitHubAPIRepositoryRelease(content))

        return contents

    async def set_last_commit(self) -> None:
        """Retrun a list of repository release objects."""
        _endpoint = f"/repos/{self.full_name}/branches/{self.default_branch}"
        response = await self.client.get(endpoint=_endpoint)
        self._last_commit = response["commit"]["sha"][0:7]

    async def get_last_commit(self) -> None:
        """Retrun a list of repository release objects."""
        _endpoint = f"/repos/{self.full_name}/branches/{self.default_branch}"
        response = await self.client.get(endpoint=_endpoint)
        return AIOGitHubAPIReposCommit(response.get("commit", {}))

    async def get_issue(self, issue: int) -> "AIOGitHubAPIRepositoryIssue":
        """Updates an issue comment."""
        _endpoint = f"/repos/{self.full_name}/issues/{issue}"

        response = await self.client.get(endpoint=_endpoint)
        return AIOGitHubAPIRepositoryIssue(self.client, response)

    async def get_issues(self) -> ["AIOGitHubAPIRepositoryIssue"]:
        """Updates an issue comment."""
        _endpoint = f"/repos/{self.full_name}/issues"

        response = await self.client.get(endpoint=_endpoint)
        return [AIOGitHubAPIRepositoryIssue(self.client, x) for x in response or []]

    async def create_issue(
        self,
        title: str or None = None,
        body: str or None = None,
        state: str or None = None,
        milestone: int or None = None,
        labels: [str] or None = None,
        assignees: [str] or None = None,
    ):
        """Updates an issue comment."""
        _endpoint = f"/repos/{self.full_name}/issues"

        data = {}
        if title is not None:
            data["title"] = title
        if body is not None:
            data["body"] = body
        if state is not None:
            data["state"] = state
        if milestone is not None:
            data["milestone"] = milestone
        if labels is not None:
            data["labels"] = labels
        if assignees is not None:
            data["assignees"] = assignees

        issue = await self.client.post(endpoint=_endpoint, data=data, jsondata=True)
        return AIOGitHubAPIRepositoryIssue(self.client, issue)



"""
Class object for AIOGitHubAPIReposCommit
Documentation: https://docs.github.com/en/rest/reference/repos#get-a-commit

Generated by generate/generate.py - 2020-08-02 15:48:55.441838
"""
from aiogithubapi.objects.base import AIOGitHubAPIBase


class CommitAuthor(AIOGitHubAPIBase):
    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def email(self):
        return self.attributes.get("email", "")

    @property
    def date(self):
        return self.attributes.get("date", "")


class CommitCommitter(AIOGitHubAPIBase):
    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def email(self):
        return self.attributes.get("email", "")

    @property
    def date(self):
        return self.attributes.get("date", "")


class CommitTree(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def sha(self):
        return self.attributes.get("sha", "")


class CommitVerification(AIOGitHubAPIBase):
    @property
    def verified(self):
        return self.attributes.get("verified", False)

    @property
    def reason(self):
        return self.attributes.get("reason", "")

    @property
    def signature(self):
        return self.attributes.get("signature", None)

    @property
    def payload(self):
        return self.attributes.get("payload", None)


class Commit(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def author(self):
        return CommitAuthor(self.attributes.get("author", {}))

    @property
    def committer(self):
        return CommitCommitter(self.attributes.get("committer", {}))

    @property
    def message(self):
        return self.attributes.get("message", "")

    @property
    def tree(self):
        return CommitTree(self.attributes.get("tree", {}))

    @property
    def comment_count(self):
        return self.attributes.get("comment_count", None)

    @property
    def verification(self):
        return CommitVerification(self.attributes.get("verification", {}))


class Author(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Committer(AIOGitHubAPIBase):
    @property
    def login(self):
        return self.attributes.get("login", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def followers_url(self):
        return self.attributes.get("followers_url", "")

    @property
    def following_url(self):
        return self.attributes.get("following_url", "")

    @property
    def gists_url(self):
        return self.attributes.get("gists_url", "")

    @property
    def starred_url(self):
        return self.attributes.get("starred_url", "")

    @property
    def subscriptions_url(self):
        return self.attributes.get("subscriptions_url", "")

    @property
    def organizations_url(self):
        return self.attributes.get("organizations_url", "")

    @property
    def repos_url(self):
        return self.attributes.get("repos_url", "")

    @property
    def events_url(self):
        return self.attributes.get("events_url", "")

    @property
    def received_events_url(self):
        return self.attributes.get("received_events_url", "")

    @property
    def type(self):
        return self.attributes.get("type", "")

    @property
    def site_admin(self):
        return self.attributes.get("site_admin", False)


class Parents(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def sha(self):
        return self.attributes.get("sha", "")


class Stats(AIOGitHubAPIBase):
    @property
    def additions(self):
        return self.attributes.get("additions", None)

    @property
    def deletions(self):
        return self.attributes.get("deletions", None)

    @property
    def total(self):
        return self.attributes.get("total", None)


class Files(AIOGitHubAPIBase):
    @property
    def filename(self):
        return self.attributes.get("filename", "")

    @property
    def additions(self):
        return self.attributes.get("additions", None)

    @property
    def deletions(self):
        return self.attributes.get("deletions", None)

    @property
    def changes(self):
        return self.attributes.get("changes", None)

    @property
    def status(self):
        return self.attributes.get("status", "")

    @property
    def raw_url(self):
        return self.attributes.get("raw_url", "")

    @property
    def blob_url(self):
        return self.attributes.get("blob_url", "")

    @property
    def patch(self):
        return self.attributes.get("patch", "")


class AIOGitHubAPIReposCommit(AIOGitHubAPIBase):
    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def html_url(self):
        return self.attributes.get("html_url", "")

    @property
    def comments_url(self):
        return self.attributes.get("comments_url", "")

    @property
    def commit(self):
        return Commit(self.attributes.get("commit", {}))

    @property
    def author(self):
        return Author(self.attributes.get("author", {}))

    @property
    def committer(self):
        return Committer(self.attributes.get("committer", {}))

    @property
    def parents(self):
        return [Parents(x) for x in self.attributes.get("parents", [])]

    @property
    def stats(self):
        return Stats(self.attributes.get("stats", {}))

    @property
    def files(self):
        return [Files(x) for x in self.attributes.get("files", [])]



"""
AIOGitHubAPI: Repository Traffic

https://docs.github.com/en/rest/reference/repos#traffic
"""
from datetime import datetime

from aiogithubapi.objects.base import AIOGitHubAPIBaseClient
from aiogithubapi.objects.repos.traffic.clones import AIOGitHubAPIReposTrafficClones
from aiogithubapi.objects.repos.traffic.pageviews import (
    AIOGitHubAPIReposTrafficPageviews,
)


class AIOGitHubAPIRepositoryTraffic(AIOGitHubAPIBaseClient):
    """Repository Release GitHub API implementation."""

    @property
    def full_name(self) -> None:
        return self.attributes.get("full_name")

    async def get_views(self):
        _endpoint = f"/repos/{self.full_name}/traffic/views"
        response = await self.client.get(endpoint=_endpoint)
        return AIOGitHubAPIReposTrafficPageviews(response)

    async def get_clones(self) -> None:
        _endpoint = f"/repos/{self.full_name}/traffic/clones"
        response = await self.client.get(endpoint=_endpoint)
        return AIOGitHubAPIReposTrafficClones(response)



"""
AIOGitHubAPI: Repository Content

https://developer.github.com/v3/repos/contents/
"""
# pylint: disable=missing-docstring
import base64

from aiogithubapi.objects.content import AIOGitHubAPIContentBase


class AIOGitHubAPIRepositoryContent(AIOGitHubAPIContentBase):
    """Repository Conetent GitHub API implementation."""

    @property
    def type(self):
        return self.attributes.get("type", "file")

    @property
    def encoding(self):
        return self.attributes.get("encoding")

    @property
    def name(self):
        return self.attributes.get("name")

    @property
    def path(self):
        return self.attributes.get("path")

    @property
    def content(self):
        return base64.b64decode(
            bytearray(self.attributes.get("content"), "utf-8")
        ).decode()

    @property
    def download_url(self):
        return self.attributes.get("download_url") or self.attributes.get(
            "browser_download_url"
        )


class AIOGitHubAPIRepositoryTreeContent(AIOGitHubAPIContentBase):
    """Repository Conetent GitHub API implementation."""

    def __init__(self, attributes, repository, ref):
        """Initialize."""
        self.attributes = attributes
        self.repository = repository
        self.ref = ref

    @property
    def full_path(self):
        return self.attributes.get("path")

    @property
    def is_directory(self):
        if self.attributes.get("type") == "tree":
            return True
        return False

    @property
    def path(self):
        path = ""
        if "/" in self.attributes.get("path"):
            path = self.attributes.get("path").split(
                f"/{self.attributes.get('path').split('/')[-1]}"
            )[0]
        return path

    @property
    def filename(self):
        filename = self.attributes.get("path")
        if "/" in self.attributes.get("path"):
            filename = self.attributes.get("path").split("/")[-1]
        return filename

    @property
    def url(self):
        return self.attributes.get("url")

    @property
    def download_url(self):
        return f"https://raw.githubusercontent.com/{self.repository}/{self.ref}/{self.attributes.get('path')}"



"""AIOGitHubAPI: AioGitHubClient"""
# pylint: disable=redefined-builtin, too-many-arguments
from asyncio import CancelledError, TimeoutError, get_event_loop
from aiohttp import ClientError

import async_timeout
import backoff

from aiogithubapi.common.exceptions import (
    AIOGitHubAPIAuthenticationException,
    AIOGitHubAPIException,
    AIOGitHubAPIRatelimitException,
)
from aiogithubapi.common.const import (
    BASE_API_HEADERS,
    BASE_API_URL,
    HTTP_STATUS_CODE_GOOD_LIST,
    HTTP_STATUS_CODE_RATELIMIT,
)
from aiogithubapi.objects.base import AIOGitHubAPIBase
from aiogithubapi.objects.ratelimit import AIOGitHubAPIRateLimit


class AIOGitHubAPIClient(AIOGitHubAPIBase):
    """Client to handle API calls."""

    def __init__(self, session: "aiohttp.ClientSession", token: str) -> None:
        """Initialize the client."""
        self.session = session
        self.token = token
        self.ratelimits = AIOGitHubAPIRateLimit()
        self.headers = BASE_API_HEADERS
        if token is not None:
            self.headers["Authorization"] = "token {}".format(token)

    async def get(
        self,
        endpoint: str,
        returnjson: bool = True,
        headers: dict or None = None,
        params: dict or None = None,
    ):
        """Execute a GET request."""
        return await self.call_api("GET", endpoint, returnjson, headers, params)

    async def post(
        self,
        endpoint: str,
        returnjson: bool = False,
        headers: dict or None = None,
        params: dict or None = None,
        data: dict or str or None = None,
        jsondata: bool = False,
    ):
        """Execute a POST request."""
        return await self.call_api(
            "POST", endpoint, returnjson, headers, params, data, jsondata
        )

    @backoff.on_exception(
        backoff.expo, (ClientError, CancelledError, TimeoutError, KeyError), max_tries=5
    )
    async def call_api(
        self,
        call: str in ["GET", "POST"],
        endpoint: str,
        returnjson: bool,
        headers: dict,
        params: dict,
        data: dict or None = None,
        jsondata: bool = False,
    ):
        """Execute the API call."""
        _url = f"{BASE_API_URL}{endpoint}"
        _headers = self.headers
        _params = {}

        for header in headers or []:
            _headers[header] = headers[header]

        for param in params or []:
            _params[param] = params[param]

        async with async_timeout.timeout(20, loop=get_event_loop()):
            if call == "GET":
                response = await self.session.get(
                    _url, params=_params, headers=_headers
                )
            else:
                if jsondata:
                    response = await self.session.post(
                        _url, params=_params, headers=_headers, json=data
                    )
                else:
                    response = await self.session.post(
                        _url, params=_params, headers=_headers, data=data
                    )
            self.logger.debug(response.headers)
            self.ratelimits.load_from_response_headers(response.headers)

            if response.status == HTTP_STATUS_CODE_RATELIMIT:
                raise AIOGitHubAPIRatelimitException("GitHub Ratelimit error")

            if response.status not in HTTP_STATUS_CODE_GOOD_LIST:
                raise AIOGitHubAPIException(
                    f"GitHub returned {response.status} for {_url}"
                )

            if returnjson:
                response = await response.json()
            else:
                response = await response.text()

            if isinstance(response, dict):
                if response.get("message"):
                    if response["message"] == "Bad credentials":
                        raise AIOGitHubAPIAuthenticationException(
                            "Access token is not valid!"
                        )
                    raise AIOGitHubAPIException(response["message"])

        self.logger.debug(response)
        return response



"""
Class object for AIOGitHubAPIReposBranch
Documentation: https://docs.github.com/en/rest/reference/repos#get-a-branch

Generated by generate/generate.py - 2020-08-02 10:29:03.550853
"""
from aiogithubapi.objects.base import AIOGitHubAPIBase


class CommitCommitAuthor(AIOGitHubAPIBase):
    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def date(self):
        return self.attributes.get("date", "")

    @property
    def email(self):
        return self.attributes.get("email", "")


class CommitCommitTree(AIOGitHubAPIBase):
    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def url(self):
        return self.attributes.get("url", "")


class CommitCommitCommitter(AIOGitHubAPIBase):
    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def date(self):
        return self.attributes.get("date", "")

    @property
    def email(self):
        return self.attributes.get("email", "")


class CommitCommitVerification(AIOGitHubAPIBase):
    @property
    def verified(self):
        return self.attributes.get("verified", False)

    @property
    def reason(self):
        return self.attributes.get("reason", "")

    @property
    def signature(self):
        return self.attributes.get("signature", None)

    @property
    def payload(self):
        return self.attributes.get("payload", None)


class CommitCommit(AIOGitHubAPIBase):
    @property
    def author(self):
        return CommitCommitAuthor(self.attributes.get("author", {}))

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def message(self):
        return self.attributes.get("message", "")

    @property
    def tree(self):
        return CommitCommitTree(self.attributes.get("tree", {}))

    @property
    def committer(self):
        return CommitCommitCommitter(self.attributes.get("committer", {}))

    @property
    def verification(self):
        return CommitCommitVerification(self.attributes.get("verification", {}))


class CommitAuthor(AIOGitHubAPIBase):
    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def login(self):
        return self.attributes.get("login", "")


class Parents(AIOGitHubAPIBase):
    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def url(self):
        return self.attributes.get("url", "")


class CommitCommitter(AIOGitHubAPIBase):
    @property
    def gravatar_id(self):
        return self.attributes.get("gravatar_id", "")

    @property
    def avatar_url(self):
        return self.attributes.get("avatar_url", "")

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def id(self):
        return self.attributes.get("id", None)

    @property
    def login(self):
        return self.attributes.get("login", "")


class Commit(AIOGitHubAPIBase):
    @property
    def sha(self):
        return self.attributes.get("sha", "")

    @property
    def node_id(self):
        return self.attributes.get("node_id", "")

    @property
    def commit(self):
        return CommitCommit(self.attributes.get("commit", {}))

    @property
    def author(self):
        return CommitAuthor(self.attributes.get("author", {}))

    @property
    def parents(self):
        return [Parents(x) for x in self.attributes.get("parents", [])]

    @property
    def url(self):
        return self.attributes.get("url", "")

    @property
    def committer(self):
        return CommitCommitter(self.attributes.get("committer", {}))


class ProtectionRequiredStatusChecks(AIOGitHubAPIBase):
    @property
    def enforcement_level(self):
        return self.attributes.get("enforcement_level", "")

    @property
    def contexts(self):
        return self.attributes.get("contexts", [])


class Protection(AIOGitHubAPIBase):
    @property
    def enabled(self):
        return self.attributes.get("enabled", True)

    @property
    def required_status_checks(self):
        return ProtectionRequiredStatusChecks(
            self.attributes.get("required_status_checks", {})
        )


class AIOGitHubAPIReposBranch(AIOGitHubAPIBase):
    @property
    def name(self):
        return self.attributes.get("name", "")

    @property
    def commit(self):
        return Commit(self.attributes.get("commit", {}))

    @property
    def protected(self):
        return self.attributes.get("protected", True)

    @property
    def protection(self):
        return Protection(self.attributes.get("protection", {}))

    @property
    def protection_url(self):
        return self.attributes.get("protection_url", "")



"""AIOGitHubAPI: objects.base"""
import logging


class AIOGitHubAPIBase:
    """Base class for AIOGitHubAPI."""

    logger = logging.getLogger("AIOGitHubAPI")

    def __init__(self, attributes) -> None:
        """Initialize."""
        self.attributes = attributes


class AIOGitHubAPIBaseClient(AIOGitHubAPIBase):
    """Base class for AIOGitHubAPI."""

    def __init__(self, client: "AIOGitHubAPIClient", attributes: dict) -> None:
        """Initialise."""
        super().__init__(attributes)
        self.client = client



"""Setup configuration."""
import setuptools

with open("README.md", "r") as fh:
    DESCRIPTION = fh.read()

setuptools.setup(
    name="aiogithubapi",
    version="1.1.2",
    author="Joakim Sorensen (@ludeeus)",
    author_email="hi@ludeeus.dev",
    description="Asynchronous Python client for the GitHub API",
    install_requires=["aiohttp>=3.6.1,<4.0", "async_timeout", "backoff"],
    long_description=DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/ludeeus/aiogithubapi",
    packages=setuptools.find_packages(
        exclude=["tests", "tests.*", "generate", "generate.*"]
    ),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



"""AIOGitHubAPI: Exceptions"""


class AIOGitHubAPIException(BaseException):
    """Raise this when something is off."""


class AIOGitHubAPIRatelimitException(AIOGitHubAPIException):
    """Raise this when we hit the ratelimit."""


class AIOGitHubAPIAuthenticationException(AIOGitHubAPIException):
    """Raise this when there is an authentication issue."""
